"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

# -*- coding: utf-8 -*-
from flask import current_app
from constants import *
from utilities.config_manager import config_manager

def get_info(cluster_obj, user=None, password=None):

    cluster_obj.get_updates(alerts_flag=True, user=user, password=password)
    info = dict()
    info['nodes'] = cluster_obj.get_node_list().values()
    info['namespaces'] = list(cluster_obj.get_namespaces(user=user, password=password))
    info['cluster_status'] = cluster_obj.get_status()
    info['indexes'] = cluster_obj.get_index_list_by_namespace(user=user, password=password)
    info['build'] = cluster_obj.get_build_no(user=user, password=password)
    info['update_interval'] = config_manager.get_update_interval()
    info['different_cluster_nodes'] = cluster_obj.get_different_cluster_nodes()
    info['cluster_builds'] = list(cluster_obj._cluster_builds)
    return info


def get_basic_info(cluster_obj, user=None, password=None):

    cluster_obj.get_updates(alerts_flag=True, user=user, password=password)
    info = dict()
    info['nodes'] = cluster_obj.get_node_list().values()
    info['off_nodes'] = cluster_obj.get_off_nodes(user=user, password=password)
    info['memory'] = cluster_obj.get_memory_usage()
    info['disk'] = cluster_obj.get_disk_usage()
    info['namespaces'] = list(cluster_obj.get_namespaces(user=user, password=password))
    info['cluster_status'] = cluster_obj.get_status()
    info['build'] = cluster_obj.get_build_no(user=user, password=password)
    info['update_interval'] = config_manager.get_update_interval()
    info['active_red_alert_count'] = cluster_obj.get_active_red_alert_count(user=user, password=password)
    #info['dc_names'] = cluster_obj.get_dc_names(user=user, password=password)
    info['build_details'] = cluster_obj.get_build_no_of_cluster_nodes()
    info['users'] = cluster_obj._users
    info['cluster_builds'] = list(cluster_obj._cluster_builds)
    version_list = info['build_details']['version_list']
    if len(version_list) == 0 or len(version_list) == 1:
        info['nodes_compatibility'] = 'homogeneous'
    else:
        min_v = max_v = version_list.keys()[0]
        flag = True
        for key in version_list.keys():
            if key < min_v:
                min_v = key
                flag = False
            elif key > max_v:
                max_v = key
                flag = False
        if flag is True:
            info['nodes_compatibility'] = 'homogeneous'
        else:
            min_v = int(str(min_v).replace(".", ""))
            max_v = int(str(max_v).replace(".", ""))
            if min_v < 354 and max_v >= 354:
                info['nodes_compatibility'] = 'incompatible'
            else:
                info['nodes_compatibility'] = 'compatible'
    return info


def get_clusters(clusters):

    clusters_info = dict(clusters=[{"id": cluster_id, "nodes": cluster_obj.get_node_list().values()}
        for cluster_id, cluster_obj in list(clusters.get_clusters().items())])
    return clusters_info


def get_throughput(cluster_obj, node_addresses, user=None, password=None):

    if node_addresses is None:
        node_addresses = cluster_obj.get_node_list().values()
    else:
        node_addresses = node_addresses.split(',')

    throughput = dict(read_tps={}, write_tps={}, cluster_status=cluster_obj.get_status())
    for node_address in node_addresses:
        node = cluster_obj.get_node(node_address)
        if node is None or node.get_status(user=user, password=password) == "off" or not node.is_valid():
            continue
        #throughput["read_tps"][node_address] = node.get_statistic("read_tps")
        #throughput["write_tps"][node_address] = node.get_statistic("write_tps")
        throughput["read_tps"][node_address] = node.get_read_tps()
        throughput["write_tps"][node_address] = node.get_write_tps()
    return throughput


def get_throughput_history(cluster_obj, node_addresses, start_time=None, end_time=None,
                                user=None, password=None):

    if node_addresses is None:
        node_addresses = cluster_obj.get_node_list().values()
    else:
        node_addresses = node_addresses.split(',')

    throughput_history = dict(read_tps={}, write_tps={}, cluster_status=cluster_obj.get_status())
    for node_address in node_addresses:
        node = cluster_obj.get_node(node_address)
        if node is None:
            continue
        throughput_history["read_tps"][node_address] = list(node.get_read_tps_history(start_time, end_time))
        throughput_history["write_tps"][node_address] = list(node.get_write_tps_history(start_time, end_time))
    return throughput_history


def get_alerts(cluster_obj, last_id, user=None, password=None):

    cluster_obj.delete_old_alerts()
    return cluster_obj.get_alerts(last_id, user=user, password=password)


def get_indexes_by_namespace(cluster_obj, namespace_name,
                                        user=None, password=None):

    result = {"cluster_status": "on"}

    indexes = cluster_obj.get_indexes_by_namespaces(namespace_name,
                                                user=user, password=password)

    if indexes == UNABLE_TO_CONNECT:
        result["cluster_status"] = "off"
        return result

    result["indexes"] = indexes
    return result


def get_sets_by_namespace(cluster_obj, namespace_name, user=None, password=None):

    result = {"cluster_status": "on"}

    sets = cluster_obj.get_sets_info_by_namespaces(namespace_name,
                                            user=user, password=password)
    if sets == UNABLE_TO_CONNECT:
        result["cluster_status"] = "off"
        return result

    result["sets"] = sets
    return result


def add_node(cluster_obj, node_address, user=None, password=None):

    return cluster_obj.add_existing_node(node_address, user=user, password=password)


def remove_node(cluster_obj, node_address):

    return cluster_obj.remove_node(node_address)


def initiate_backup(cluster_obj, namespace, destination_node_address,
destination_location, sets, only_metadata, scan_priority, terminate_on_change,
username="", password=None, usr=None, passwd=None, pl_password=None):

    return cluster_obj.initiate_backup(namespace, destination_node_address,
    username, password, destination_location, sets, only_metadata,
    scan_priority, terminate_on_change, usr=usr, passwd=passwd, pl_password=pl_password)


def get_backup_progress(cluster_obj):

    return cluster_obj.get_backup_progress()


def get_successful_backups(cluster_obj):

    return cluster_obj.get_successful_backups()


def get_available_backups(cluster_obj, source_node_address, source_location,
username, password):

    return cluster_obj.get_available_backups(source_node_address,
    source_location, username, password)


def initiate_restore(cluster_obj, source_node_address, source_location, threads,
namespace, missing_records_only, ignore_generation_num, username="",
password=None, usr=None, passwd=None, pl_password=None):

    return cluster_obj.initiate_restore(source_node_address, source_location,
    threads, namespace, missing_records_only, ignore_generation_num, username,
    password, usr=usr, passwd=passwd, pl_password=pl_password)


def get_restore_progress(cluster_obj, user=None, password=None):

    return cluster_obj.get_restore_progress(user=user, password=password)


def get_user_roles(cluster_obj, user, passwd):

    """
    Get user roles for a user on a agiven cluster
    """
    roles = cluster_obj.get_user_roles(user, passwd)
    if roles is None : return []
    return roles

def add_user(cluster_obj, user, passwd, roles):

    response = cluster_obj.add_new_user(user, passwd, roles)
    return response


def remove_user(cluster_obj, user):

    response = cluster_obj.drop_user(user)
    return response


def update_user(cluster_obj, user, roles, old_roles, passwd=None):

    response, status = cluster_obj.update_user(user, roles, old_roles, passwd)
    return response, status


def get_all_users(cluster_obj):

    response = cluster_obj.get_all_users()
    return response


def change_password(cluster_obj, user, old_passwd, new_passwd):

    response = cluster_obj.change_password(user, old_passwd, new_passwd)
    return response


def close_sockets(cluster_obj):

    cluster_obj.close_sockets()

def get_all_roles(cluster_obj):

    response = cluster_obj.get_all_roles()
    return response

def get_role(cluster_obj, role):
    
    response = cluster_obj.get_role(role)
    return response    

def add_role(cluster_obj, role, privileges):
    
    response = cluster_obj.add_role(role, privileges)
    return response

def drop_role(cluster_obj, role):

    response = cluster_obj.drop_role(role)
    return response

def add_privileges(cluster_obj, role, privileges):

    response = cluster_obj.add_privileges(role, privileges)
    return response

def drop_privileges(cluster_obj, role, privileges):

    response = cluster_obj.drop_privileges(role, privileges)
    return response

def update_role(cluster_obj, role, privileges, old_privileges):

    response = cluster_obj.update_role(role, privileges, old_privileges)
    return response

def fire_cmd(cluster_obj, cmd, user=None, password=None):

    return cluster_obj.fire_cmd(cmd, user=user, password=password)